{
  TECHNISOFT Group presents:

  FILES SCANNER V2.03b (c) 1998 by Alexander Burlakov aka Hunter
                           E-mail: Hunter@hptmts.tvcom.ru
                           URL: http://www.chat.ru/~hunterml/

  This is freeware beta version. If you will use this component or have
  any comments, please, contact with me (E-mail: Hunter@hptmts.tvcom.ru).

  UNIT FILESSCN.PAS includs:

        TFilesScan    - abstract base class for creating Files scanners.
                        UpdateFilesList method, declared in public part of
                        this class, is used for update Files lists in user
                        defined children comps.

        TFilesScanner - full functional non visual component that perform
                        fast files scan (using multiple file mask for example:
                        *.bak, *.old, [0-1]*.tmp ). Using TFilesList - my new
                        extended List with fast sorting (by 3 ways) feature.

        It's easy to use (see source).

  Nearest future projects & upgrades:
       * TDBFilesScanner - component for posting scanned files into
                           TDataSet.
       * TFilesCScan     - NEW abstract class like TFilesScan, but also
                           scanning files for specified context.
       (for more information contact via E-mail).
}
unit FilesScn;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms, Fllist;

type
  TScanOptions = (soGlobal, soSpecifiedDisk, soSpecifiedDir);
  TFileAttribute = (faReadOnly, faHidden, faSysFile, faVolumeID,
                    faDirectory, faArchive);
  TFileAttributes = set of TFileAttribute;
  TOnProcessEvent = procedure (Sender: TObject; Dir : string;
                       FilesCount, FilesSize : longint) of object;
  TOnChangedEvent = procedure (Sender: TObject;
                       FilesCount, FilesSize : longint) of object;


  TFilesScan = class(TComponent)
  private
    { Private declarations }
    FMask : string;
    FAttr : TFileAttributes;
    FUpdateList : boolean;
    FScanOptions : TScanOptions;
    FScanDir : string;
    FRecursive : boolean;
    FProcessMsg : boolean;
    FFilesCount : longint;
    FFilesSize : longint;
    FOnProcess : TOnProcessEvent;
    FOnChanged : TOnChangedEvent;
    FStopScan : boolean;
    procedure SetScanDir(const Value : string);
    procedure ScanDirectory(const ADir : string; const AAttr : integer;
                            AMasks : TCollection);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    procedure Scan; virtual;
    procedure Stop; virtual;
    procedure UpdateFilesList(const ADir: string; AFile : TSearchRec); virtual; abstract;
    function GetIntegerAttr : integer;
    property FilesCount : longint read FFilesCount;
    property FilesSize : longint read FFilesSize;
  published
    { Published declarations }
    property FileMask : string read FMask write FMask;
    property Attributes : TFileAttributes read FAttr write FAttr stored true;
    property UpdateList : boolean read FUpdateList write FUpdateList;
    property ScanOptions : TScanOptions read FScanOptions write FScanOptions;
    property ScanDir : string read FScanDir write SetScanDir;
    property ProcessMsg : boolean read FProcessMsg write FProcessMsg;
    property Recursive : boolean read FRecursive write FRecursive;
    property OnProcess : TOnProcessEvent read FOnProcess write FOnProcess;
    property OnChanged : TOnChangedEvent read FOnChanged write FOnChanged;
  end;

  TFilesScanner = class(TFilesScan)
  private
    FFiles : TFilesList;
    procedure SetSorted(const Value : TSortOptions);
    function GetSorted : TSortOptions;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Scan; override;
    procedure UpdateFilesList(const ADir: string; AFile : TSearchRec); override;
    property Files : TFilesList read FFiles;
 published
    property Sorted : TSortOptions read GetSorted write SetSorted;
  end;

function GetFileAttrStr(Attr : integer) : string;

procedure Register;

implementation

uses FileCtrl, Masks;

type
  TMultiMask = class(TCollectionItem)
    FMask : TMask;
  public
    destructor Destroy; override;
  end;

function GetFileAttrStr(Attr : integer) : string;
begin
  Result := '';
  if (Attr and SysUtils.faArchive) <> 0 then Result := Result+'A'
     else Result := Result+'-';
  if (Attr and SysUtils.faDirectory) <> 0 then Result := Result+'D'
     else Result := Result+'-';
  if (Attr and SysUtils.faHidden) <> 0 then Result := Result+'H'
     else Result := Result+'-';
  if (Attr and SysUtils.faReadOnly) <> 0 then Result := Result+'R'
     else Result := Result+'-';
  if (Attr and SysUtils.faSysFile) <> 0 then Result := Result+'S'
     else Result := Result+'-';
  if (Attr and SysUtils.faVolumeID) <> 0 then Result := Result+'V'
     else Result := Result+'-';
end;

function SlashSep(const Path, S: String): String;
begin
  Result := '';
  if Path = '' then Exit;
  if AnsiLastChar(Path)^ <> '\' then
    Result := Path + '\' + S
  else
    Result := Path + S;
end;

procedure CreateMasks(ACol : TCollection; AStr : string);
var
  P : TMultiMask;
  i : integer;
  S : string;
begin
  i := 1; S := '';
  while i <= Length(AStr) do
  begin
    case AnsiChar(AStr[i]) of
     ' ' : begin
             if S = '' then Inc(i) else
             begin
               S := S + AStr[i];
               if i = Length(AStr) then
               begin
                 while AnsiLastChar(S)^ = ' ' do
                     S := Copy(S, 1, Length(S)-1);
                 P := TMultiMask(ACol.Add);
                 P.FMask := TMask.Create(S);
                 Exit;
               end else  Inc(i);
             end;
           end;
     ',' : begin
             while AnsiLastChar(S)^ = ' ' do
                  S := Copy(S, 1, Length(S)-1);
             P := TMultiMask(ACol.Add);
             P.FMask := TMask.Create(S);
             S := '';
             if i = Length(AStr) then Exit else Inc(i);
           end;
    else   begin
             S := S + AStr[i];
             if i = Length(AStr) then
             begin
               while AnsiLastChar(S)^ = ' ' do
                    S := Copy(S, 1, Length(S)-1);
               P := TMultiMask(ACol.Add);
               P.FMask := TMask.Create(S);
               Exit;
             end else  Inc(i);
           end;
    end;
  end;
end;

{ TMultiMask }

destructor TMultiMask.Destroy;
begin
  if Assigned(FMask) then FMask.Free;
  inherited;
end;

{ TFilesScan }

constructor TFilesScan.Create;
begin
  inherited Create(AOwner);
  FMask := '*';
  FScanDir := 'C:\';
  FAttr := [faArchive];
  FUpdateList := true;
  FScanOptions := soGlobal;
  FProcessMsg := true; FRecursive := true;
  FStopScan := false;
end;

function TFilesScan.GetIntegerAttr : integer;
begin
  Result := 0;
  if faReadOnly in FAttr then Result := Result + SysUtils.faReadOnly;
  if faHidden in FAttr then Result := Result + SysUtils.faHidden;
  if faSysFile in FAttr then Result := Result + SysUtils.faSysFile;
  if faVolumeID in FAttr then Result := Result + SysUtils.faVolumeID;
  if faDirectory in FAttr then Result := Result + SysUtils.faDirectory;
  if faArchive in FAttr then Result := Result + SysUtils.faArchive;
end;

procedure TFilesScan.SetScanDir;
begin
  FScanDir := Value;
  if (csDesigning in ComponentState) then
  begin
    if FScanDir <> '' then FScanOptions := soSpecifiedDir else
       FScanOptions := soGlobal;
    if (FScanDir <> '') and (Length(FScanDir) < 3) and (AnsiLastChar(FScanDir)^ = ':') then
        FScanOptions := soSpecifiedDisk;
  end;
end;

procedure TFilesScan.Scan;
var
  Masks : TCollection;

  procedure GlobalScan;
  var
    DriveNum: Integer;
    DriveChar: Char;
    DriveType: TDriveType;
    DriveBits: set of 0..25;
  begin
    Integer(DriveBits) := GetLogicalDrives;
    for DriveNum := 0 to 25 do
    begin
      if not (DriveNum in DriveBits) then Continue;
      DriveChar := Char(DriveNum + Ord('a'));
      DriveType := TDriveType(GetDriveType(PChar(DriveChar + ':\')));
      DriveChar := Upcase(DriveChar);
      case DriveType of
        dtFixed, dtNetwork,
        dtCDROM, dtRAM: ScanDirectory(DriveChar+':', GetIntegerAttr, Masks);
      end;
    end;
  end;
begin
  FFilesCount := 0; FFilesSize := 0;
  Masks := TCollection.Create(TMultiMask);
  CreateMasks(Masks, FMask);
  FStopScan := false;
  case FScanOptions of
    soGlobal : GlobalScan;
    soSpecifiedDisk : ScanDirectory(Copy(FScanDir, 1, 2), GetIntegerAttr, Masks);
    soSpecifiedDir : ScanDirectory(FScanDir, GetIntegerAttr, Masks);
  end;
  Masks.Destroy;
  if Assigned(FOnChanged) then
     FOnChanged(Self, FFilesCount, FFilesSize);
end;

procedure TFilesScan.ScanDirectory;
var
  Status: Integer;
  SearchRec: TSearchRec;

  function MultiMatches(N : string) : boolean;
  var i : integer;
  begin
    MultiMatches := true;
    for i := 0 to Pred(AMasks.Count) do
        if TMultiMask(AMasks.Items[i]).FMask.Matches(N) then exit;
    MultiMatches := false;
  end;

begin
  if Assigned(FOnProcess) then
     FOnProcess(Self, ADir, FFilesCount, FFilesSize);
  Status := FindFirst(SlashSep(ADir, '*.*'), faAnyFile, SearchRec);
  try
    while (Status = 0) and (not FStopScan) do
    begin
      if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') then
      begin
        if (SearchRec.Attr and AAttr = AAttr) and
           MultiMatches(SearchRec.Name) then
        begin
          Inc(FFilesCount); Inc(FFilesSize, SearchRec.Size);
          if FUpdateList then
             UpdateFilesList(ADir, SearchRec); // <= This is NEW feature:
                // virtual procedure for Appending userdefined Files List
         end;
        if FRecursive and (SearchRec.Attr and SysUtils.faDirectory = SysUtils.faDirectory) then
            ScanDirectory(SlashSep(ADir, SearchRec.Name), AAttr, AMasks);
      end;
      Status := FindNext(SearchRec);
      if FProcessMsg then Application.ProcessMessages;
    end;
  finally
    FindClose(SearchRec);
  end;
end;

procedure TFilesScan.Stop;
begin
  FStopScan := true;
end;

{ TFilesScanner }

constructor TFilesScanner.Create;
begin
  inherited Create(AOwner);
  FFiles := TFilesList.Create;
  with FFiles do Sorted := soUnsort;
end;

destructor TFilesScanner.Destroy;
begin
  FFiles.Free;
  inherited Destroy;
end;

procedure TFilesScanner.Scan;
begin
  FFiles.Clear;
  inherited Scan;
end;

procedure TFilesScanner.UpdateFilesList;
var F : TFileObject;
begin
  F := TFileObject.Create(AFile.Name, SlashSep(ADir, ''), AFile.Size,
       AFile.Attr, AFile.Time);
  FFiles.Add(F);
end;

procedure TFilesScanner.SetSorted;
begin
  Files.Sorted := Value;
  if Assigned(OnChanged) and (Files.Count <> 0) then
     OnChanged(Self, FilesCount, FilesSize);
end;

function TFilesScanner.GetSorted;
begin
  Result := Files.Sorted;
end;

procedure Register;
begin
  RegisterComponents('Technisoft', [TFilesScanner]);
end;

end.
