{
  TECHNISOFT Group presents:

  FILES SCANNER V2.01b (c) 1998 by Alexander Burlakov aka Hunter
                           E-mail: Hunter@hptmts.tvcom.ru
                           URL: http://www.chat.ru/~hunterml/

  This is freeware beta version. If you will use this component or have
  any comments, please, contact with me (E-mail: Hunter@hptmts.tvcom.ru).


  UNIT FLLIST.PAS  includs:

  TFileObject  - special object to storing files
                 information in TFilesList.

  TFilesList   - Fast sorted TList like component for
                 storing files information (such as
                 Name, Size, Time, Full Path);
}
unit Fllist;

interface

uses Classes;

type

  TSortOptions = (soUnsort, soByName, soBySize, soByTime);

  TFileObject = class(TObject)
    FileName : string;
    FullPath : string;
    FileSize : integer;
    FileAttr : integer;
    FileTime : integer;
  public
    constructor Create(AName, APath : string; ASize, AAttr, ATime: integer); virtual;
  end;

{ Overriding TList existing methods for future using }
  TBaseFileList = class(TList)
  public
    function Add(Item: Pointer): Integer; virtual; abstract;
    procedure Delete(Index : integer); virtual; abstract;
    procedure Clear; virtual; abstract;
    procedure Sort; virtual; abstract;
  end;

  TFilesList = class(TBaseFileList)
  private
    FSortOptions : TSortOptions;
    procedure SetSortOptions(const Value : TSortOptions);
    procedure QuickSort(L, R: Integer);
  public
    function Add(Item: Pointer): Integer; override;
    procedure Clear; override;
    function Compare(Item1, Item2: Pointer): Integer; virtual;
    procedure Delete(Index : integer); override;
    function Find(S: pointer; var Index: Integer): Boolean; virtual;
    procedure Sort; override;
  published
    property Sorted: TSortOptions read FSortOptions write SetSortOptions;
  end;

implementation
uses SysUtils;

function IntCompare(I1, I2 : integer): integer;
begin
  Result := 0;
  if I1 < I2 then Result := 1;
  if Result = 0 then if I1 > I2 then Result := -1;
end;

{ TFileObject }

constructor TFileObject.Create;
begin
  inherited Create;
  FileName := AName;
  FullPath := APath;
  FileSize := ASize;
  FileAttr := AAttr;
  FileTime := ATime;
end;

{ TFilesList }

function TFilesList.Add;
begin
  if FSortOptions = soUnsort then Result := Count
     else Find(Item, Result);
  Insert(Result, Item);
end;

procedure TFilesList.Clear;
begin
  while Count <> 0 do Delete(0);
  Capacity := 0;
end;

function TFilesList.Compare(Item1, Item2: Pointer): Integer;
begin
  case FSortOptions of
   soByName : Result := AnsiCompareFileName(TFileObject(Item1).FileName,
                                 TFileObject(Item2).FileName);
   soBySize : Result := IntCompare(TFileObject(Item1).FileSize,
                                 TFileObject(Item2).FileSize);
   soByTime : Result := IntCompare(TFileObject(Item1).FileTime,
                                 TFileObject(Item2).FileTime);
  else Result := 0;
  end;
end;

procedure TFilesList.Delete;
var
  P : pointer;
begin
  if (Index < 0) or (Index >= Count) then exit;
  P := List[Index];
  if Assigned(P) then TFileObject(P).Destroy;
  Count := Count - 1;
  if Index < Count then
    System.Move(List^[Index + 1], List^[Index],
      (Count - Index) * SizeOf(Pointer));
end;

function TFilesList.Find;
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := Count - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    C := Compare(List^[I], S);
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
         Result := True;
    end;
  end;
  Index := L;
end;

procedure TFilesList.QuickSort(L, R: Integer);
var
  I, J: Integer;
  P, T: Pointer;
begin
  repeat
    I := L;
    J := R;
    P := List^[(L + R) shr 1];
    repeat
      while Compare(List^[I], P) < 0 do Inc(I);
      while Compare(List^[J], P) > 0 do Dec(J);
      if I <= J then
      begin
        T := List^[I];
        List^[I] := List^[J];
        List^[J] := T;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(L, J);
    L := I;
  until I >= R;
end;

procedure TFilesList.SetSortOptions;
begin
  if Value <> FSortOptions then
  begin
    FSortOptions := Value;
    Sort;
  end;
end;

procedure TFilesList.Sort;
begin
  if (List <> nil) and (Count > 0) then
    QuickSort(0, Count - 1);
end;

end.
